@@ -39,7 +39,7 @@ struct _EphySQLiteStatement {
   EphySQLiteConnection *connection;
 };
 
-G_DEFINE_TYPE (EphySQLiteStatement, ephy_sqlite_statement, G_TYPE_OBJECT);
+G_DEFINE_FINAL_TYPE (EphySQLiteStatement, ephy_sqlite_statement, G_TYPE_OBJECT);
 
 static void
 ephy_sqlite_statement_set_property (GObject      *object,
@@ -304,7 +304,8 @@ ephy_sqlite_create_match_pattern (const char *match_string)
   char *string, *pattern;
 
   string = g_strndup (match_string, EPHY_SQLITE_LIMIT_LIKE_PATTERN_LENGTH - 2);
-  pattern = g_strdup_printf ("%%:%%%s%%", string);
+  /* Create a search pattern: '%string%' which will be used in sqlite search statement */
+  pattern = g_strdup_printf ("%%%s%%", string);
   g_free (string);
 
   return pattern;
