@@ -22,12 +22,12 @@
 #pragma once
 
 #include "ephy-window.h"
-#include <gtk/gtk.h>
+#include <adwaita.h>
 
 G_BEGIN_DECLS
 
 #define EPHY_TYPE_ENCODING_DIALOG (ephy_encoding_dialog_get_type ())
-G_DECLARE_FINAL_TYPE (EphyEncodingDialog, ephy_encoding_dialog, EPHY, ENCODING_DIALOG, GtkWindow);
+G_DECLARE_FINAL_TYPE (EphyEncodingDialog, ephy_encoding_dialog, EPHY, ENCODING_DIALOG, AdwWindow);
 
 EphyEncodingDialog  *ephy_encoding_dialog_new  (EphyWindow *window);
 
