@@ -1468,7 +1468,7 @@ ephy_application_dialog_data_free (EphyApplicationDialogData *data)
     g_signal_handlers_disconnect_by_func (data->download, download_finished_cb, data);
     g_signal_handlers_disconnect_by_func (data->download, download_failed_cb, data);
 
-    data->download = NULL;
+    g_clear_object (&data->download);
   }
 
   g_cancellable_cancel (data->cancellable);
@@ -1791,6 +1791,7 @@ set_default_application_title (EphyApplicationDialogData *data,
   }
 
   if (title == NULL || title[0] == '\0') {
+    g_clear_pointer (&title, g_free);
     title = g_strdup (webkit_web_view_get_title (WEBKIT_WEB_VIEW (data->view)));
   }
 
@@ -1951,7 +1952,7 @@ static void
 create_install_dialog_when_ready (EphyApplicationDialogData *data)
 {
   XdpPortal *portal;
-  XdpParent *parent;
+  g_autoptr (XdpParent) parent = NULL;
 
   if (!data->webapp_options_set || !data->title || !data->icon_v)
     return;
@@ -2918,6 +2919,7 @@ clipboard_text_received_cb (GdkClipboard *clipboard,
   EphyWebView *web_view;
   g_autoptr (GError) error = NULL;
   g_autofree char *text = NULL;
+  g_autofree char *normalized = NULL;
 
   text = gdk_clipboard_read_text_finish (clipboard, res, &error);
   if (error) {
@@ -2925,6 +2927,8 @@ clipboard_text_received_cb (GdkClipboard *clipboard,
     return;
   }
 
+  normalized = ephy_embed_utils_normalize_or_autosearch_address (text);
+
   embed = ephy_embed_container_get_active_child (EPHY_EMBED_CONTAINER (window));
   g_assert (embed != NULL);
 
@@ -2934,7 +2938,7 @@ clipboard_text_received_cb (GdkClipboard *clipboard,
                               0);
 
   web_view = ephy_embed_get_web_view (embed);
-  ephy_web_view_load_url (web_view, text);
+  ephy_web_view_load_url (web_view, normalized);
 
   ephy_embed_container_set_active_child (EPHY_EMBED_CONTAINER (window), embed);
   gtk_widget_grab_focus (GTK_WIDGET (embed));
